package fr.nitorac.aurionweb.aurionweb;

import android.content.Context;
import android.util.TypedValue;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Toast;

import androidx.core.content.ContextCompat;

import com.afollestad.materialdialogs.LayoutMode;
import com.afollestad.materialdialogs.MaterialDialog;
import com.afollestad.materialdialogs.bottomsheets.BottomSheet;
import com.afollestad.materialdialogs.bottomsheets.BottomSheetsKt;
import com.afollestad.materialdialogs.customview.DialogCustomViewExtKt;

import org.jsoup.Connection;
import org.jsoup.nodes.Document;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.function.Consumer;

import fr.nitorac.aurionweb.R;
import fr.nitorac.aurionweb.StorageManager;
import fr.nitorac.aurionweb.Utils;
import fr.nitorac.aurionweb.security.SecurityManager;

public class AurionManager {

    private static AurionManager instance;

    public AurionManager(){

    }



    public boolean checkPassword(AurionSession session, AurionCred cred) throws GeneralSecurityException {
        if(session == null){
            session = new AurionSession();
        }

        if(cred.getLogin().isEmpty() || cred.getPlainPwd().isEmpty()){
            return false;
        }

        try {
            initConnection(session, cred.getLogin(), cred.getPlainPwd());
            return true;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    public Document reachMyEdtFromInit(AurionSession session) throws IOException, GeneralSecurityException {
        // On charge une fois la page pour valider un affichage fictif (cote server aurion)
        // Requête effective permettant la validation de l'authentification sur CAS
        Connection.Response resp = Utils.wrapSSL("https://aurionweb.ensiie.fr/faces/MainMenuPage.xhtml")
                .method(Connection.Method.POST)
                .followRedirects(true)
                .header("Cache-Control", "max-age=0")
                .header("Content-Type", "application/x-www-form-urlencoded")
                .header("Host", "aurionweb.ensiie.fr")
                .header("Origin", "https://aurionweb.ensiie.fr")
                .header("Referer", "https://aurionweb.ensiie.fr/")
                .header("Upgrade-Insecure-Requests", "1")
                .header("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36")
                .cookie("JSESSIONID", session.getJSESSIONID())
                .data("form", "form")
                //.data("form:largeurDivCenter", "1219")
                //.data("form:sauvegarde", "")
                .data("form:sidebar", "form:sidebar")
                .data("form:sidebar_menuid", session.getSubmenuId())
                .data("javax.faces.ViewState", session.getViewState())
                .execute();
        Document respDoc = resp.parse();
        session.parseViewState(respDoc);

        return respDoc;
    }

    // Instancie une connection vers AurionWeb
    public Document initConnection(AurionSession session, String username, String pwd) throws IOException, GeneralSecurityException {
        session.clear();
        // Première connection pour récupérer un String "execution" nécessaire à la connection
        Connection.Response initRes = Utils.wrapSSL("https://cas.ensiie.fr/login?service=https%3A%2F%2Faurionweb.ensiie.fr%2F%2Flogin%2Fcas")
                .method(Connection.Method.GET)
                .header("Cache-Control", "max-age=0")
                .header("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36")
                .execute();
        String execution = initRes.parse().getElementsByAttributeValue("name", "execution").get(0).attr("value");
        //Requête effective permettant la validation de l'authentification sur CAS
        Connection secConn = Utils.wrapSSL("https://cas.ensiie.fr/login?service=https%3A%2F%2Faurionweb.ensiie.fr%2F%2Flogin%2Fcas")
                .method(Connection.Method.POST)
                .followRedirects(true)
                .header("Cache-Control", "max-age=0")
                .header("Content-Type", "application/x-www-form-urlencoded")
                .header("Host", "cas.ensiie.fr")
                .header("Origin", "https://cas.ensiie.fr")
                .header("Referer", "https://cas.ensiie.fr/login?service=https%3A%2F%2Faurionweb.ensiie.fr%2F%2Flogin%2Fcas")
                .header("Sec-Fetch-Mode", "navigate")
                .header("Sec-Fetch-Site", "same-origin")
                .header("Upgrade-Insecure-Requests", "1")
                .header("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36")
                .data("username", username)
                .data("_eventId", "submit")
                //.data("geolocation", "")
                .data("execution", execution);
        secConn.data("password", pwd);
        Connection.Response sec = secConn.execute();
        Document doc1 = sec.parse();
        session.parseViewState(doc1);
        session.setJSESSIONID(sec.cookie("JSESSIONID"));
        //On trouve le form ID
        session.parseConnectMenuFormId(doc1);
        //On trouve l'ID du bouton Emploi du temps (pour les requêtes)
        session.parseConnectMenuId(doc1);
        //Requête effective permettant la validation de l'authentification sur CAS
        Connection.Response tmp = Utils.wrapSSL("https://aurionweb.ensiie.fr/faces/MainMenuPage.xhtml")
                .method(Connection.Method.POST)
                .followRedirects(true)
                .header("Cache-Control", "max-age=0")
                .header("Content-Type", "application/x-www-form-urlencoded")
                .header("Host", "aurionweb.ensiie.fr")
                .header("Origin", "https://aurionweb.ensiie.fr")
                .header("Referer", "https://aurionweb.ensiie.fr/")
                .header("Faces-Request", "partial/ajax")
                .header("Upgrade-Insecure-Requests", "1")
                .header("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36")
                .cookie("JSESSIONID", session.getJSESSIONID())
                .data("javax.faces.partial.ajax", "true")
                .data("javax.faces.source", session.getMenuFormId())
                .data("javax.faces.partial.execute", session.getMenuFormId())
                .data("javax.faces.partial.render", "form:sidebar")
                .data(session.getMenuFormId(), session.getMenuFormId())
                .data("webscolaapp.Sidebar.ID_SUBMENU", session.getMenuId())
                .data("form", "form")
                //.data("form:largeurDivCenter", "1219")
                //.data("form:sauvegarde", "")
                .data("javax.faces.ViewState", session.getViewState())
                .execute();
        Document doc2 = tmp.parse();
        session.parseViewState(doc2);
        session.parseConnectSubmenuId(doc2);

        return doc2;
    }

    public void spawnLoginDialog(Context c){
        spawnLoginDialog(c, (ignored) -> {});
    }

    public void spawnLoginDialog(Context c, Consumer<Void> callback) {
        final MaterialDialog dialog = new MaterialDialog(c, new BottomSheet(LayoutMode.MATCH_PARENT))
                .title(null, "Connexion AurionWeb")
                .icon(null, ContextCompat.getDrawable(c, R.drawable.ic_login))
                .cancelable(true)
                .noAutoDismiss()
                .negativeButton(null, "ANNULER", materialDialog -> {
                    materialDialog.dismiss();
                    return null;
                });
        BottomSheetsKt.setPeekHeight(dialog, (int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 400 /*dp*/, c.getResources().getDisplayMetrics()), null);

        // Errk, dirtyyyy, but I can't find any other way to do it
        DialogCustomViewExtKt.customView(dialog, R.layout.dialog_login, null, false, false, true, true);

        final EditText login = dialog.findViewById(R.id.loginInput);
        final EditText pwd = dialog.findViewById(R.id.passwordInput);
        final CheckBox remember = dialog.findViewById(R.id.rememberMe);

        final StringBuilder savedPwd = new StringBuilder();

        try{
            AurionCred cred = StorageManager.getInstance().getCreds(c);
            if(cred != null){
                login.setText(cred.getLogin());
                savedPwd.append(cred.getPlainPwd());
                pwd.setHint("(aucune modification)");
                remember.setChecked(cred.isRemember());
            }
        }catch (Exception e){
            e.printStackTrace();
            Toast.makeText(c, "Erreur : " + e.getLocalizedMessage(), Toast.LENGTH_LONG).show();
        }

        dialog.positiveButton(null, "CONNEXION", materialDialog -> {
            final AurionCred newCred = new AurionCred();
            newCred.setLogin(login.getText().toString().toLowerCase().trim());

            String tmpPwd = pwd.getText().toString().trim();

            if(!savedPwd.toString().isEmpty() && tmpPwd.isEmpty()){
                tmpPwd = savedPwd.toString();
            }

            newCred.setPlainPwd(tmpPwd);
            newCred.setRemember(remember.isChecked());

            CheckConnectionTask checkTask = new CheckConnectionTask(c, (res) -> {
                if(res){
                    Toast.makeText(c, "Connexion réussie !", Toast.LENGTH_LONG).show();
                    materialDialog.dismiss();
                    SecurityManager.creds = newCred;
                    try{
                        StorageManager.getInstance().saveCreds(c, newCred.isRemember() ? newCred : new AurionCred());
                    }catch (Exception e){
                        Toast.makeText(c, "Impossible de sauver les paramètres (Erreur : " + e.getLocalizedMessage() + ")", Toast.LENGTH_LONG).show();
                    }
                    callback.accept(null);
                }else{
                    Toast.makeText(c, "Mauvais mot de passe ou erreur de connexion !", Toast.LENGTH_LONG).show();
                }
            });
            checkTask.execute(newCred);
            return null;
        });

        // Errk, dirtyyyy, workaround to avoid a flickering issue when dialog is sliding
        dialog.dismiss();

        dialog.show();
    }

    public static AurionManager getInstance(){
        if(instance == null){
            instance = new AurionManager();
        }

        return instance;
    }
}
